# --------------------------------------------------------------  
# (C)Copyright 2001,2006,                                         
# International Business Machines Corporation,                    
# Sony Computer Entertainment, Incorporated,                      
# Toshiba Corporation,                                            
#                                                                 
# All Rights Reserved.                                            
#                                                                 
# Redistribution and use in source and binary forms, with or      
# without modification, are permitted provided that the           
# following conditions are met:                                   
#                                                                 
# - Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer. 
#                                                                 
# - Redistributions in binary form must reproduce the above       
#   copyright notice, this list of conditions and the following   
#   disclaimer in the documentation and/or other materials        
#   provided with the distribution.                               
#                                                                 
# - Neither the name of IBM Corporation nor the names of its      
#   contributors may be used to endorse or promote products       
#   derived from this software without specific prior written     
#   permission.                                                   
#                                                                 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND          
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,     
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR            
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT    
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;    
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)        
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN       
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR    
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              
# --------------------------------------------------------------  
# PROLOG END TAG zYx                                              

########################################################################
#			Target
########################################################################

# Determine the target processor by the subdirectory where the
# library is being built (e.g. ppu, ppu64, spu).

TARGET_PROCESSOR	:= $(notdir $(CURDIR))

LIBRARY			:= libgmath.a
SHARED_LIBRARY		:= libgmath.so


########################################################################
#			Local Defines
########################################################################

INCLUDE			:= -I../../vector -I../../misc


########################################################################
#			Objects
########################################################################

INLINE_BUILT_OBJS	:= cos8.o		cos8_v.o		\
			   cos14.o		cos14_v.o		\
			   cos18.o		cos18_v.o		\
			   pack_normal16.o	pack_normal16_v.o	\
						pack_rgba8_v.o		\
			   sin8.o		sin8_v.o		\
			   sin14.o		sin14_v.o		\
			   sin18.o		sin18_v.o		\
			   spec9.o		spec9_v.o		\
			   unpack_normal16.o	unpack_normal16_v.o	\
			   unpack_rgba8.o	unpack_rgba8_v.o

NON_INLINE_BUILT_OBJS	:= pack_color8.o				\
			   pack_rgba8.o					\
			   set_spec_exponent9.o				\
			   unpack_color8.o

OBJS			:= $(INLINE_BUILT_OBJS)				\
			   $(NON_INLINE_BUILT_OBJS)

ifeq "$(TARGET_PROCESSOR)" "spu"
  C_FROM_LIBRARY_AWK	:= $(OBJS:.o=.c)
else
  C_FROM_LIBRARY_AWK	:= $(INLINE_BUILT_OBJS:.o=.c)
  C_FROM_LIBRARY2_AWK	:= $(NON_INLINE_BUILT_OBJS:.o=.c)
endif


########################################################################
#			Install
########################################################################

INSTALL_FILES		:= libgmath.h			\
			   cos_sin8.h	cos_sin8_v.h	\
			   cos_sin14.h	cos_sin14_v.h	\
			   cos_sin18.h	cos_sin18_v.h	\
			   normal16.h			\
			   spec9_types.h		\
			   $(patsubst %.o,%.h, $(OBJS))


########################################################################
#			Common Lib Rules
########################################################################

include ../../lib.mk
