/* --------------------------------------------------------------  */
/* (C)Copyright 2001,2007,                                         */
/* International Business Machines Corporation,                    */
/* Sony Computer Entertainment, Incorporated,                      */
/* Toshiba Corporation,                                            */
/*                                                                 */
/* All Rights Reserved.                                            */
/*                                                                 */
/* Redistribution and use in source and binary forms, with or      */
/* without modification, are permitted provided that the           */
/* following conditions are met:                                   */
/*                                                                 */
/* - Redistributions of source code must retain the above copyright*/
/*   notice, this list of conditions and the following disclaimer. */
/*                                                                 */
/* - Redistributions in binary form must reproduce the above       */
/*   copyright notice, this list of conditions and the following   */
/*   disclaimer in the documentation and/or other materials        */
/*   provided with the distribution.                               */
/*                                                                 */
/* - Neither the name of IBM Corporation nor the names of its      */
/*   contributors may be used to endorse or promote products       */
/*   derived from this software without specific prior written     */
/*   permission.                                                   */
/*                                                                 */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND          */
/* CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,     */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        */
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        */
/* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR            */
/* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    */
/* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT    */
/* NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;    */
/* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)        */
/* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN       */
/* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR    */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              */
/* --------------------------------------------------------------  */
/* PROLOG END TAG zYx                                              */
#include <stdlib.h>
#include <stdio.h>
#include "liblarge_matrix.h"
//#define TRANSPOSE_SCALAR 1

/*
 * Description:
 *    transpose matrix mat
 *
 * Parameters:
 *    m: number of rows in matrix a and cols in b
 *    n: number of cols in matrix a and rows in b
 *    a: pointer to matrix. Matrix must be quad-word aligned
 *    lda: leading dimension of matrix a
 *    b: pointer to matrix b, matrix must be quad-word aligned 
 *    ldb: leading dimesion of matrix b
 *
 * NOTE: array a and b must be quadword aligned with number of rows 
 *       and cols multiples of 4
 * 
 * Returns:
 *    the tranposed matrix is returned in b 
 */
void transpose_matrix (int m, int n, float* a, int lda, float* b, int ldb)
{

#ifdef TRANSPOSE_SCALAR 
  int i, j;
  
  for (j = 0; j < n; j++) {
    for (i = 0; i < m; i++) {
      b[j*ldb + i] = a[i*lda + j]; 
    }
  }
#else
  int i, j;
  vector float abcd, efgh, ijkl, mnop;	/* input vectors */
  vector float aeim, bfjn, cgko, dhlp;	/* output vectors */
  vector float aibj, ckdl, emfn, gohp;	/* intermediate vectors */
  vector unsigned char shufflehi = (vector unsigned char) {
					       0x00, 0x01, 0x02, 0x03, 0x10, 0x11, 0x12, 0x13,
					       0x04, 0x05, 0x06, 0x07, 0x14, 0x15, 0x16, 0x17};
  vector unsigned char shufflelo = (vector unsigned char) {
					       0x08, 0x09, 0x0A, 0x0B, 0x18, 0x19, 0x1A, 0x1B,
					       0x0C, 0x0D, 0x0E, 0x0F, 0x1C, 0x1D, 0x1E, 0x1F};

  for (i =0; i < m; i+= 4) {
    for (j = 0; j < n; j+=4) {
      
      abcd = *((vector float*) (&a[i*lda + j]));
      efgh = *((vector float*) (&a[(i+1)*lda + j]));
      ijkl = *((vector float*) (&a[(i+2)*lda + j]));
      mnop = *((vector float*) (&a[(i+3)*lda + j]));

      aibj = spu_shuffle(abcd, ijkl, shufflehi);
      ckdl = spu_shuffle(abcd, ijkl, shufflelo);
      emfn = spu_shuffle(efgh, mnop, shufflehi);
      gohp = spu_shuffle(efgh, mnop, shufflelo);

      aeim = spu_shuffle(aibj, emfn, shufflehi);
      bfjn = spu_shuffle(aibj, emfn, shufflelo);
      cgko = spu_shuffle(ckdl, gohp, shufflehi);
      dhlp = spu_shuffle(ckdl, gohp, shufflelo);
  
      *((vector float*) (&b[j*ldb + i])) = aeim;
      *((vector float*) (&b[(j+1)*ldb + i])) = bfjn;
      *((vector float*) (&b[(j+2)*ldb + i])) = cgko;
      *((vector float*) (&b[(j+3)*ldb + i])) = dhlp;
    }
  }
#endif /* TRANSPOSE_SCALAR */
}
