# --------------------------------------------------------------  
# (C)Copyright 2001,2006,                                         
# International Business Machines Corporation,                    
# Sony Computer Entertainment, Incorporated,                      
# Toshiba Corporation,                                            
#                                                                 
# All Rights Reserved.                                            
#                                                                 
# Redistribution and use in source and binary forms, with or      
# without modification, are permitted provided that the           
# following conditions are met:                                   
#                                                                 
# - Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer. 
#                                                                 
# - Redistributions in binary form must reproduce the above       
#   copyright notice, this list of conditions and the following   
#   disclaimer in the documentation and/or other materials        
#   provided with the distribution.                               
#                                                                 
# - Neither the name of IBM Corporation nor the names of its      
#   contributors may be used to endorse or promote products       
#   derived from this software without specific prior written     
#   permission.                                                   
#                                                                 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND          
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,     
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR            
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT    
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;    
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)        
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN       
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR    
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              
# --------------------------------------------------------------  
# PROLOG END TAG zYx                                              

########################################################################
#			Target
########################################################################

# Determine the target processor by the subdirectory where the
# library is being built (e.g. ppu, ppu64, spu).

TARGET_PROCESSOR	:= $(notdir $(CURDIR))

LIBRARY			:= libmatrix.a
SHARED_LIBRARY		:= libmatrix.so


########################################################################
#			Local Defines
########################################################################

INCLUDE			:= -I../../misc


########################################################################
#			Objects
########################################################################

INLINE_BUILT_OBJS	:= frustum_matrix4x4.o		\
			   identity_matrix4x4.o		\
			   inverse_matrix4x4.o		\
			   mult_matrix4x4.o		\
			   mult_quat.o			\
			   ortho_matrix4x4.o		\
			   perspective_matrix4x4.o	\
			   quat_to_rot_matrix4x4.o	\
			   rot_matrix4x4_to_quat.o	\
			   scale_matrix4x4.o		\
			   splat_matrix4x4.o		\
			   transpose_matrix4x4.o

NON_INLINE_BUILT_OBJS	:= rotate_matrix4x4.o		\
			   slerp_quat.o

COMMON_OBJS		:= $(INLINE_BUILT_OBJS)		\
			   $(NON_INLINE_BUILT_OBJS)


ifeq "$(TARGET_PROCESSOR)" "spu"
  TARGET_OBJS		:= mult_matrix4x4_d.o		\
			   cast_matrix4x4_to_dbl.o 	\
			   cast_matrix4x4_to_flt.o
endif

OBJS			:= $(COMMON_OBJS) $(TARGET_OBJS)


ifeq "$(TARGET_PROCESSOR)" "spu"
  C_FROM_LIBRARY_AWK	:= $(OBJS:.o=.c)
else
  C_FROM_LIBRARY_AWK	:= $(INLINE_BUILT_OBJS:.o=.c)
  C_FROM_LIBRARY2_AWK	:= $(NON_INLINE_BUILT_OBJS:.o=.c)
endif


########################################################################
#			Install
########################################################################

INSTALL_FILES	   := libmatrix.h				\
		      $(patsubst %.o,%.h, $(COMMON_OBJS))	\
		      $(patsubst %.o,%.h, $(TARGET_OBJS))


########################################################################
#			Common Lib Rules
########################################################################

include ../../lib.mk
