# --------------------------------------------------------------  
# (C)Copyright 2001,2008,                                         
# International Business Machines Corporation,                    
# Sony Computer Entertainment, Incorporated,                      
# Toshiba Corporation,                                            
#                                                                 
# All Rights Reserved.                                            
#                                                                 
# Redistribution and use in source and binary forms, with or      
# without modification, are permitted provided that the           
# following conditions are met:                                   
#                                                                 
# - Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer. 
#                                                                 
# - Redistributions in binary form must reproduce the above       
#   copyright notice, this list of conditions and the following   
#   disclaimer in the documentation and/or other materials        
#   provided with the distribution.                               
#                                                                 
# - Neither the name of IBM Corporation nor the names of its      
#   contributors may be used to endorse or promote products       
#   derived from this software without specific prior written     
#   permission.                                                   
#                                                                 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND          
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,     
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR            
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT    
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;    
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)        
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN       
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR    
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              
# --------------------------------------------------------------  
# PROLOG END TAG zYx                                              

########################################################################
#			Target
########################################################################

# Determine the target processor by the subdirectory where the
# library is being built (e.g. ppu, ppu64, spu).

TARGET_PROCESSOR	:= $(notdir $(CURDIR))

LIBRARY			:= libmisc.a
SHARED_LIBRARY		:= libmisc.so


########################################################################
#			Objects
########################################################################

INLINE_BUILT_OBJS	:= calloc_align.o				\
			   clamp_0_to_1.o	  clamp_0_to_1_v.o	\
			   clamp_minus1_to_1.o	  clamp_minus1_to_1_v.o	\
			   clamp.o	   	  clamp_v.o		\
			   free_align.o					\
			   load_vec_unaligned.o				\
			   malloc_align.o				\
			   min_float_v.o	  max_float_v.o		\
			   min_int_v.o	   	  max_int_v.o		\
						  rand_v.o		\
			   rand_minus1_to_1.o	  rand_minus1_to_1_v.o	\
			   rand_0_to_1.o	  rand_0_to_1_v.o	\
			   realloc_align.o				\
						  srand_v.o		\
			   store_vec_unaligned.o

NON_INLINE_BUILT_OBJS	:= min_vec_float3.o	  max_vec_float3.o	\
			   min_vec_float4.o	  max_vec_float4.o	\
			   min_vec_int3.o	  max_vec_int3.o	\
			   min_vec_int4.o	  max_vec_int4.o

TARGET_OBJS		:= rand_seed.o

ifeq "$(TARGET_PROCESSOR)" "spu"
  C_FROM_LIBRARY_AWK	 = $(HEADER_OBJS:.o=.c)
  TARGET_OBJS		+= cpex.o
  INSTALL_FILES		:= cpex.h

  # uncomment the following line to build cpex copy functions to be interrupt safe
  # CPPFLAGS		+= -DSPU_MFCIO_INTERRUPT_SAFE
else
  C_FROM_LIBRARY_AWK	:= $(INLINE_BUILT_OBJS:.o=.c)
  C_FROM_LIBRARY2_AWK	:= $(NON_INLINE_BUILT_OBJS:.o=.c)
endif

HEADER_OBJS	 	 = $(INLINE_BUILT_OBJS)	$(NON_INLINE_BUILT_OBJS)

OBJS			:= $(HEADER_OBJS) $(TARGET_OBJS)


########################################################################
#			Install
########################################################################

INSTALL_FILES		+= libmisc.h			\
			   rand_util.h			\
			   $(patsubst %.o,%.h, $(HEADER_OBJS))


########################################################################
#			Common Lib Rules
########################################################################

include ../../lib.mk
