/* --------------------------------------------------------------  */
/* (C)Copyright 2006,2008,                                         */
/* International Business Machines Corporation                     */
/* All Rights Reserved.                                            */
/*                                                                 */
/* Redistribution and use in source and binary forms, with or      */
/* without modification, are permitted provided that the           */
/* following conditions are met:                                   */
/*                                                                 */
/* - Redistributions of source code must retain the above copyright*/
/*   notice, this list of conditions and the following disclaimer. */
/*                                                                 */
/* - Redistributions in binary form must reproduce the above       */
/*   copyright notice, this list of conditions and the following   */
/*   disclaimer in the documentation and/or other materials        */
/*   provided with the distribution.                               */
/*                                                                 */
/* - Neither the name of IBM Corporation nor the names of its      */
/*   contributors may be used to endorse or promote products       */
/*   derived from this software without specific prior written     */
/*   permission.                                                   */
/*                                                                 */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND          */
/* CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,     */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        */
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        */
/* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR            */
/* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    */
/* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT    */
/* NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;    */
/* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)        */
/* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN       */
/* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR    */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              */
/* --------------------------------------------------------------  */
/* PROLOG END TAG zYx                                              */
#include <race_check.h>
#include "race_check_global.h"

/* race_check_access
 * =================
 * Verify that the specified local store access does not produce a race 
 * condition with previously issued DMAs. The local store region being
 * accessed is specified by the start and size parameters.
 *
 * The local store access is specified by the parameters:
 *  start - specifies the starting local store address being accessed.
 *  size  - specifies the number of bytes in the region being accessed.
 *  access_type - specifies whether the region is being read, written
 *          or both read and written.
 *  filename - pointer to the name of the file that initiated the access.
 *          This optional parameter is only used when RACE_CHECK_VERBOSE 
 *          is defined.
 *  line  - file line number that initiated the access. This optional 
 *          parameter is only used when RACE_CHECK_VERBOSE is defined.
 */

void race_check_access(void *start, 
		       unsigned int size, 
		       enum access_types access_type
		       RACE_CHECK_VERBOSE_PARMS1)
{
  int i;
  unsigned int flags;

#if RACE_CHECK_VERBOSE && RACE_CHECK_DEBUG
  fprintf(RACE_CHECK_STREAM, "RACE CHECK - %s (%s:%d start=%p size=0x%x)\n", __PRETTY_FUNCTION__, filename, line, start, size);
#endif

  if (size != 0) {
    for (i=0; i<RACE_CHECK_DB_SIZE; i++) {
      flags = race_check_db[i].flags;

      if (flags & FLAG_INFLIGHT) {
	if ((start < (race_check_db[i].start + race_check_db[i].size)) &&
	    (race_check_db[i].start < (start+size)) && 
	    (race_check_db[i].size) &&
	    ((access_type & RACE_CHECK_WRITE) || (IS_GET_CMD(race_check_db[i].cmd)))) {
#if RACE_CHECK_VERBOSE
	  fprintf(RACE_CHECK_STREAM, "RACE CHECK ERROR detected for access in %s:%d (start=%p size=0x%x) with a DMA issued in %s:%d (cmd=start=%p size=0x%x)\n",
		  filename, line, start, size, race_check_db[i].filename, race_check_db[i].line, race_check_db[i].start, race_check_db[i].size);
#else
	  fprintf(RACE_CHECK_STREAM, race_check_format, RACE_CHECK_ACCESS_ERROR);
#endif
	}
	if (flags & FLAG_DMA_LIST) {
	  if ((start < ((void *)race_check_db[i].list + race_check_db[i].list_size)) &&
	      ((void *)race_check_db[i].list < (start+size)) &&
	      (race_check_db[i].list_size) &&
	      (access_type & RACE_CHECK_WRITE)) {
#if RACE_CHECK_VERBOSE
	    fprintf(RACE_CHECK_STREAM, "RACE CHECK ERROR detected for access in %s:%d (start=%p size=0x%x) with a list array of the DMA issued in %s:%d (cmd=start=%p size=0x%x)\n",
		    filename, line, start, size, race_check_db[i].filename, race_check_db[i].line, race_check_db[i].start, race_check_db[i].size);
#else
	    fprintf(RACE_CHECK_STREAM, race_check_format, RACE_CHECK_ACCESS_LIST_ERROR);
#endif
	  }
	}
      }
    }
  }
}
