/* --------------------------------------------------------------  */
/* (C)Copyright 2006,2008,                                         */
/* International Business Machines Corporation                     */
/* All Rights Reserved.                                            */
/*                                                                 */
/* Redistribution and use in source and binary forms, with or      */
/* without modification, are permitted provided that the           */
/* following conditions are met:                                   */
/*                                                                 */
/* - Redistributions of source code must retain the above copyright*/
/*   notice, this list of conditions and the following disclaimer. */
/*                                                                 */
/* - Redistributions in binary form must reproduce the above       */
/*   copyright notice, this list of conditions and the following   */
/*   disclaimer in the documentation and/or other materials        */
/*   provided with the distribution.                               */
/*                                                                 */
/* - Neither the name of IBM Corporation nor the names of its      */
/*   contributors may be used to endorse or promote products       */
/*   derived from this software without specific prior written     */
/*   permission.                                                   */
/*                                                                 */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND          */
/* CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,     */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        */
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        */
/* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR            */
/* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    */
/* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT    */
/* NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;    */
/* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)        */
/* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN       */
/* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR    */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              */
/* --------------------------------------------------------------  */
/* PROLOG END TAG zYx                                              */
#include <spu_mfcio.h>
#include <race_check.h>
#include "race_check_global.h"

/* race_check_barrier
 * ==================
 * Called when a MFC_BARRIER command is issued by marking all DMA's in the 
 * race check data base as being barriered for the tag group specified by
 * the tag parameter Then when a wait is performed on that tag, all marked 
 * DMAs can be removed from the database so a race condition is not flagged 
 * for a memory access on that transferred region.
 *
 * tag - specified the tag ID the MFC_BARRIER command is issued for.
 */

void race_check_barrier(unsigned int tag)
{
  int i;
  int entry;

#if RAVE_CHECK_VERBOSE && RACE_CHECK_DEBUG
  fprintf(RACE_CHECK_STREAM, "RACE CHECK - %s (tag=%d)\n", __PRETTY_FUNCTION__, tag);
#endif

  for (i=0; i<RACE_CHECK_DB_SIZE; i++) {
    /* The barrier command only ensure ordering of queued commands */
    if (IS_QUEUED_CMD(race_check_db[i].cmd)) {
      race_check_db[i].flags |= FLAG_BARRIERED_NON_TAG_SPECIFIC;
      race_check_db[i].barrier_tags |= 1 << tag;
    }
  }
  /* Add the barrier command to the database. It is needed to support
   * the correct race check resumption of stalled DMA lists.
   */
  entry = _race_check_find_db_entry();

  race_check_db[entry].cmd = MFC_BARRIER_CMD;
  race_check_db[entry].size = 0;
  race_check_db[entry].tag = tag;
  race_check_db[entry].flags = FLAG_INFLIGHT;
}
