/* --------------------------------------------------------------  */
/* (C)Copyright 2006,2008,                                         */
/* International Business Machines Corporation                     */
/* All Rights Reserved.                                            */
/*                                                                 */
/* Redistribution and use in source and binary forms, with or      */
/* without modification, are permitted provided that the           */
/* following conditions are met:                                   */
/*                                                                 */
/* - Redistributions of source code must retain the above copyright*/
/*   notice, this list of conditions and the following disclaimer. */
/*                                                                 */
/* - Redistributions in binary form must reproduce the above       */
/*   copyright notice, this list of conditions and the following   */
/*   disclaimer in the documentation and/or other materials        */
/*   provided with the distribution.                               */
/*                                                                 */
/* - Neither the name of IBM Corporation nor the names of its      */
/*   contributors may be used to endorse or promote products       */
/*   derived from this software without specific prior written     */
/*   permission.                                                   */
/*                                                                 */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND          */
/* CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,     */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        */
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        */
/* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR            */
/* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    */
/* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT    */
/* NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;    */
/* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)        */
/* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN       */
/* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR    */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              */
/* --------------------------------------------------------------  */
/* PROLOG END TAG zYx                                              */
#include <spu_mfcio.h>
#include <race_check.h>
#include "race_check_global.h"

/* _race_check_add_db_entry
 * ========================
 * Internal function used to locate a empty race checker data base entry.
 *
 * Entries are maitained in the database ordered from oldest to newest.
 * Therefore, the data base entries must be moved to accomodate space
 * if the end of the data base is not empty.
 *
 * If there are no empty entries, then the old entry is discarded and
 * an error is issued indicating a database overflow.
 */
int _race_check_find_db_entry()
{
  int i;
  int last_empty = -1;
  int last_occupied = -1;
  unsigned int flags;

  /* Scan the database to find a open entry
   */
  for (i=0; i<RACE_CHECK_DB_SIZE; i++) {
    flags = race_check_db[i].flags;

    if (flags & (FLAG_INFLIGHT | FLAG_STALLED)) {
      last_occupied = i;
    } else {
      last_empty = i;
    }
  }

  /* There are no trailing empty entries. Move them up to open an entry
   * at the end of the database.
   */
  if (last_occupied == (RACE_CHECK_DB_SIZE-1)) {
    /* If there were no empty entries, issue a data base overflow
     * error.
     */
    if (last_empty == -1) {
#if RACE_CHECK_VERBOSE
      fprintf(RACE_CHECK_STREAM, race_check_overflow, RACE_CHECK_DB_SIZE);
#else
      fprintf(RACE_CHECK_STREAM, race_check_format, RACE_CHECK_DATA_BASE_OVERFLOW);
#endif
      last_empty = 0;
    }

    for (; last_empty < RACE_CHECK_DB_SIZE-1; last_empty++) {
      race_check_db[last_empty] = race_check_db[last_empty+1];
    }
    last_occupied--;
  } 

  return (last_occupied+1);
}
