/* --------------------------------------------------------------  */
/* (C)Copyright 2006,2008,                                         */
/* International Business Machines Corporation                     */
/* All Rights Reserved.                                            */
/*                                                                 */
/* Redistribution and use in source and binary forms, with or      */
/* without modification, are permitted provided that the           */
/* following conditions are met:                                   */
/*                                                                 */
/* - Redistributions of source code must retain the above copyright*/
/*   notice, this list of conditions and the following disclaimer. */
/*                                                                 */
/* - Redistributions in binary form must reproduce the above       */
/*   copyright notice, this list of conditions and the following   */
/*   disclaimer in the documentation and/or other materials        */
/*   provided with the distribution.                               */
/*                                                                 */
/* - Neither the name of IBM Corporation nor the names of its      */
/*   contributors may be used to endorse or promote products       */
/*   derived from this software without specific prior written     */
/*   permission.                                                   */
/*                                                                 */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND          */
/* CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,     */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        */
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        */
/* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR            */
/* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    */
/* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT    */
/* NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;    */
/* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)        */
/* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN       */
/* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR    */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              */
/* --------------------------------------------------------------  */
/* PROLOG END TAG zYx                                              */
#ifndef _RACE_CHECK_GLOBAL_H_
#define _RACE_CHECK_GLOBAL_H_	1

#include <race_check.h>

/* Race check data base.
 */
typedef struct _race_check_db_entry {
#if RACE_CHECK_VERBOSE
  const char *filename __attribute__ ((aligned (16)));	/* file name that initiated the transfer */
  int line;			/* line number that initiated the transfer */
#endif
  void *start;			/* pointer to the start of the region */
  void *list;			/* list element array */
  unsigned int flags;		/* see below */
  unsigned short size;		/* number of bytes transfered */
  unsigned short list_size;	/* size, in bytes, of the list array through stall */
  unsigned short req_list_size;	/* if stallable, requested requested list size left */
  unsigned char cmd;		/* transfer command */
  unsigned char tag;		/* tag group of the transfer */
  unsigned int barrier_tags;	/* ids in which a barrier has been issued */
} race_check_db_entry;


/* Flag bits 
 */
#define FLAG_INFLIGHT			1	/* Issued, not complete or stalled */
#define FLAG_DMA_LIST			2	/* DMA is a list form */

#define FLAG_UPDATING			4	/* tag status update is in progress */
#define FLAG_STALLABLE			8	/* DMA list contains a stall, but not yet notified */
#define FLAG_STALLED			16	/* DMA list contains a stall, that has been notified */

#define FLAG_BARRIERED_TAG_SPECIFIC	32
#define FLAG_BARRIERED_NON_TAG_SPECIFIC	64
#define FLAG_BARRIERED			(FLAG_BARRIERED_TAG_SPECIFIC | FLAG_BARRIERED_NON_TAG_SPECIFIC)



extern race_check_db_entry race_check_db[RACE_CHECK_DB_SIZE];

#if RACE_CHECK_VERBOSE
extern char * race_check_overflow;
#else
extern char * race_check_format;
#endif

extern unsigned int race_check_tag_mask;

#define IS_QUEUED_CMD(_cmd)			(((_cmd) & 0x98) != 0x90)	/* All but getllar, putllc, and putlluc */
#define IS_NON_QUEUED_ATOMIC_CMD(_cmd)		(((_cmd) & 0x98) == 0x90)	/* getllar, putllc, and putlluc */
#define IS_GET_CMD(_cmd)			((_cmd) & 0x40)			/* Command is a get, getf, getb, getl, getlf, getlb, or getllar */
#define IS_GETL_CMD(_cmd)			(((_cmd) & 0x44) == 0x44)	/* Command is a getl, getlf, getlb, or getllar */
#define IS_EITHER_GET_CMD(_cmd1, _cmd2)		(((_cmd1) | (_cmd2)) & 0x40)	/* Either command is a get variant */
#define IS_TAG_SPECIFIC_BARRIER_CMD(_cmd)	(((_cmd) & 0xC8) == 0xC8)	/* mfceieio or mfcsync command */

/* Internal race check utilities */

extern int _race_check_find_db_entry();

#endif /* _RACE_CHECK_GLOBAL_H_ */
