/* --------------------------------------------------------------  */
/* (C)Copyright 2006,2008,                                         */
/* International Business Machines Corporation                     */
/* All Rights Reserved.                                            */
/*                                                                 */
/* Redistribution and use in source and binary forms, with or      */
/* without modification, are permitted provided that the           */
/* following conditions are met:                                   */
/*                                                                 */
/* - Redistributions of source code must retain the above copyright*/
/*   notice, this list of conditions and the following disclaimer. */
/*                                                                 */
/* - Redistributions in binary form must reproduce the above       */
/*   copyright notice, this list of conditions and the following   */
/*   disclaimer in the documentation and/or other materials        */
/*   provided with the distribution.                               */
/*                                                                 */
/* - Neither the name of IBM Corporation nor the names of its      */
/*   contributors may be used to endorse or promote products       */
/*   derived from this software without specific prior written     */
/*   permission.                                                   */
/*                                                                 */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND          */
/* CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,     */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        */
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        */
/* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR            */
/* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    */
/* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT    */
/* NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;    */
/* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)        */
/* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN       */
/* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR    */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              */
/* --------------------------------------------------------------  */
/* PROLOG END TAG zYx                                              */
#include <spu_mfcio.h>
#include <race_check.h>
#include "race_check_global.h"

/* race_check_list_stall_notify
 * ============================
 * For each of the tag groups specified in the status parameter, find the first 
 * (oldest) stallable DMA list and mark it stalled. This means that the list completed
 * up through the next element with stall and notify set and can no longer be considered
 * as a participant in a race condition.
 */

void race_check_list_stall_notify(unsigned int status)
{
  int i;
  unsigned int flags;
  unsigned int tag;
  
#if RACE_CHECK_VERBOSE && RACE_CHECK_DEBUG
  fprintf(RACE_CHECK_STREAM, "RACE CHECK - %s (status=0x%x)\n", __PRETTY_FUNCTION__, status);
#endif

  /* Search the race check database for the first stallable DMA list in the tag groups
   * that have stalled.
   */
  for (i=0; i<RACE_CHECK_DB_SIZE; i++) {
    flags = race_check_db[i].flags;
    tag   = race_check_db[i].tag;

    if ((flags & FLAG_STALLABLE) && (status & (1U<<tag))) {
      flags &= ~(FLAG_STALLABLE | FLAG_INFLIGHT);
      flags |= FLAG_STALLED;
      race_check_db[i].flags = flags;
      status &= ~(1U<<tag);
      if (status == 0) break;
    }
  }
}
