/* --------------------------------------------------------------  */
/* (C)Copyright 2006,2008,                                         */
/* International Business Machines Corporation                     */
/* All Rights Reserved.                                            */
/*                                                                 */
/* Redistribution and use in source and binary forms, with or      */
/* without modification, are permitted provided that the           */
/* following conditions are met:                                   */
/*                                                                 */
/* - Redistributions of source code must retain the above copyright*/
/*   notice, this list of conditions and the following disclaimer. */
/*                                                                 */
/* - Redistributions in binary form must reproduce the above       */
/*   copyright notice, this list of conditions and the following   */
/*   disclaimer in the documentation and/or other materials        */
/*   provided with the distribution.                               */
/*                                                                 */
/* - Neither the name of IBM Corporation nor the names of its      */
/*   contributors may be used to endorse or promote products       */
/*   derived from this software without specific prior written     */
/*   permission.                                                   */
/*                                                                 */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND          */
/* CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,     */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        */
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        */
/* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR            */
/* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    */
/* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT    */
/* NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;    */
/* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)        */
/* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN       */
/* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR    */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              */
/* --------------------------------------------------------------  */
/* PROLOG END TAG zYx                                              */
#include <spu_mfcio.h>
#include <race_check.h>
#include "race_check_global.h"

/* race_check_tag_specific_barrier
 * ===============================
 * Called when a MFC_SYNC, MFC_EIEIO, and DMA command with
 * barrier (ie, getb, putb, getlb, putlb, sndsigb) command 
 * is issued. The tag parameter specifies tag group the tag 
 * specific barrier is issued against. All DMAs in the same 
 * tag group are marked as being barriered so that a future
 * DMA in this tag group will not be detected as a race
 * condition.
 */

void race_check_tag_specific_barrier(unsigned int tag)
{
  int i;

#if RACE_CHECK_VERBOSE && RACE_CHECK_DEBUG
  fprintf(RACE_CHECK_STREAM, "RACE CHECK - %s (tag=%d)\n", __PRETTY_FUNCTION__, tag);
#endif

  /* Mark all DMA's with the same tag a being barried.
   */
  for (i=0; i<RACE_CHECK_DB_SIZE; i++) {
    if ((race_check_db[i].tag == tag) && IS_QUEUED_CMD(race_check_db[i].cmd)) {
      race_check_db[i].flags |= FLAG_BARRIERED_TAG_SPECIFIC;
    }
  }
}
