/* --------------------------------------------------------------  */
/* (C)Copyright 2006,2008,                                         */
/* International Business Machines Corporation                     */
/* All Rights Reserved.                                            */
/*                                                                 */
/* Redistribution and use in source and binary forms, with or      */
/* without modification, are permitted provided that the           */
/* following conditions are met:                                   */
/*                                                                 */
/* - Redistributions of source code must retain the above copyright*/
/*   notice, this list of conditions and the following disclaimer. */
/*                                                                 */
/* - Redistributions in binary form must reproduce the above       */
/*   copyright notice, this list of conditions and the following   */
/*   disclaimer in the documentation and/or other materials        */
/*   provided with the distribution.                               */
/*                                                                 */
/* - Neither the name of IBM Corporation nor the names of its      */
/*   contributors may be used to endorse or promote products       */
/*   derived from this software without specific prior written     */
/*   permission.                                                   */
/*                                                                 */
/* THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND          */
/* CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,     */
/* INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        */
/* MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        */
/* DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR            */
/* CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    */
/* SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT    */
/* NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;    */
/* LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)        */
/* HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN       */
/* CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR    */
/* OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  */
/* EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              */
/* --------------------------------------------------------------  */
/* PROLOG END TAG zYx                                              */
#include <spu_mfcio.h>
#include <race_check.h>
#include "race_check_global.h"

/* race_check_tag_status
 * =====================
 * Wait for DMA completion of all DMA tag groups specified by the tag_status
 * parameter. DMAs with the tag group specified in the tag_status parameter
 * are removed from the race check DMA database.
 *
 * tag_status - specifies the tag groups that have been completed. The lsb
 *              corresponds to tag group 0. The msb corresponds to tag group
 *              31.
 */

void race_check_tag_status(unsigned int tag_status)
{
  int i;
  unsigned int flags;
  
#if RACE_CHECK_VERBOSE && RACE_CHECK_DEBUG
  fprintf(RACE_CHECK_STREAM, "RACE CHECK - %s (tag_status=0x%x)\n", __PRETTY_FUNCTION__, tag_status);
#endif

  tag_status &= race_check_tag_mask;

  /* Zero out the race check data base entries for all completed
   * DMAs with the same tag IDs specified in the tag mask or
   * that are marked as barriered by a tag ID in the mask.
   */
  for (i=0; i<RACE_CHECK_DB_SIZE; i++) {
    flags = race_check_db[i].flags;

    if (flags & FLAG_UPDATING) {
      flags &= ~FLAG_UPDATING;

      if ((tag_status & (1 << race_check_db[i].tag)) |
	  (race_check_db[i].barrier_tags & tag_status)) {
	flags = 0;
      }
      race_check_db[i].flags = flags;
    }
  }
}
