# --------------------------------------------------------------  
# (C)Copyright 2001,2008,                                         
# International Business Machines Corporation,                    
# Sony Computer Entertainment, Incorporated,                      
# Toshiba Corporation,                                            
#                                                                 
# All Rights Reserved.                                            
#                                                                 
# Redistribution and use in source and binary forms, with or      
# without modification, are permitted provided that the           
# following conditions are met:                                   
#                                                                 
# - Redistributions of source code must retain the above copyright
#   notice, this list of conditions and the following disclaimer. 
#                                                                 
# - Redistributions in binary form must reproduce the above       
#   copyright notice, this list of conditions and the following   
#   disclaimer in the documentation and/or other materials        
#   provided with the distribution.                               
#                                                                 
# - Neither the name of IBM Corporation nor the names of its      
#   contributors may be used to endorse or promote products       
#   derived from this software without specific prior written     
#   permission.                                                   
#                                                                 
# THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND          
# CONTRIBUTORS "AS IS" AND ANY EXPRESS OR IMPLIED WARRANTIES,     
# INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF        
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE        
# DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT OWNER OR            
# CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,    
# SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT    
# NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;    
# LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)        
# HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN       
# CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR    
# OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE,  
# EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.              
# --------------------------------------------------------------  
# PROLOG END TAG zYx                                              

########################################################################
#			Target
########################################################################

# Determine the target processor by the subdirectory where the
# library is being built (e.g. ppu, ppu64, spu).

TARGET_PROCESSOR	:= $(notdir $(CURDIR))
LIBRARY			:= libsync.a
SHARED_LIBRARY		:= libsync.so


########################################################################
#			Local Defines
########################################################################

ifeq "$(TARGET_PROCESSOR:_trace=)" "spu"
VPATH			+= ../spu_source
else
VPATH			+= ../ppu_source
endif

########################################################################
#			Objects
########################################################################

INLINE_BUILT_OBJS	:= atomic_add.o			\
			   atomic_add_return.o		\
			   atomic_dec_and_test.o	\
			   atomic_dec.o			\
			   atomic_dec_if_positive.o	\
			   atomic_dec_return.o		\
			   atomic_inc.o			\
			   atomic_inc_return.o		\
			   atomic_read.o		\
			   atomic_set.o			\
			   atomic_sub_and_test.o	\
			   atomic_sub.o			\
			   atomic_sub_return.o		\
			   cond_broadcast.o		\
			   cond_init.o			\
			   cond_signal.o		\
			   cond_wait.o			\
			   mutex_init.o			\
			   mutex_lock.o			\
			   mutex_trylock.o		\
			   mutex_unlock.o

ifeq "$(TARGET_PROCESSOR:_trace=)" "spu"
  INLINE_BUILT_OBJS	+= read_lock.o			\
			   read_trylock.o		\
			   read_unlock.o		\
			   rwlock_init.o		\
			   write_lock.o			\
			   write_trylock.o		\
			   write_unlock.o
endif

COMMON_OBJS		:= $(INLINE_BUILT_OBJS)		\
			   $(NON_INLINE_BUILT_OBJS)

OBJS			:= $(COMMON_OBJS) $(TARGET_OBJS)


ifeq "$(TARGET_PROCESSOR:_trace=)" "spu"
  C_FROM_LIBRARY_AWK	:= $(OBJS:.o=.c)

  # uncomment the following line to build sync functions to be interrupt safe
  # CPPFLAGS		+= -DSPU_MFCIO_INTERRUPT_SAFE
else
  C_FROM_LIBRARY_AWK	:= $(INLINE_BUILT_OBJS:.o=.c)
  C_FROM_LIBRARY2_AWK	:= $(NON_INLINE_BUILT_OBJS:.o=.c)
endif


########################################################################
#			Install
########################################################################

INSTALL_FILES		= atomic.h				\
			  cond.h				\
			  mutex.h				\
			  libsync.h				\
			  trace_libsync.h			\
			  sync_utils.h				\
			  $(patsubst %.o,%.h, $(COMMON_OBJS))	\
			  $(patsubst %.o,$(TARGET_PROCESSOR)/%.h, $(TARGET_OBJS))

ifeq "$(TARGET_PROCESSOR:_trace=)" "spu"
  INSTALL_FILES		+= sync_irq.h			
endif

ifneq "$(TARGET_PROCESSOR:_trace=)" "$(TARGET_PROCESSOR)"
INCLUDE			+= -I$(SDKINC)/trace
CFLAGS			+= -DLIBSYNC_TRACE -D_LEVEL=1
ifeq "$(TARGET_PROCESSOR:_trace=)" "spu"
IMPORTS			 = ${SDKLIB}/trace/libtrace.a
else # ppu
ifneq "$(TARGET_PROCESSOR:_shared=)" "$(TARGET_PROCESSOR)"
LDFLAGS			+= -ltrace -L${SDKLIB}/trace
else # static
IMPORTS			 = ${SDKLIB}/trace/libtrace.a
endif
endif
endif

########################################################################
#			Common Lib Rules
########################################################################

include ../../lib.mk
